## TODO: Change names of functions to make sure they don't clash
## with others in the tool

get_times <- function(fcst.horizon, leadtimes, sdate) {

  ## TODO: Adapt to Auto-S2S recipe format
  switch(fcst.horizon,
	 ## TODO: Remove "sub_obs"?
	 ## TODO: daily case?
         "seasonal" = {len <- length(leadtimes); ref <- 'months since '; 
                       stdname <- paste(strtoi(leadtimes), collapse=", ")},
         "sub_obs" = {len <- 52; ref <- 'week of the year ';
                      stdname <- paste(strtoi(leadtimes), collapse=", ")},
         "subseasonal" = {len <- 4; ref <- 'weeks since ';
                          stdname <- ''}
  )

  time <- 1:len
  dim(time) <- length(time)
  #metadata <- list(time = list(standard_name = stdname,
  metadata <- list(time = list(units = paste0(ref, sdate, ' 00:00:00')))
  attr(time, 'variables') <- metadata
  names(dim(time)) <- 'time'
  
  time_step <- 1
  dim(time_step) <- length(time_step)
  metadata <- list(time_step = list(units = paste0(ref, sdate, ' 00:00:00')))
  attr(time_step, 'variables') <- metadata
  names(dim(time_step)) <- 'time_step'

  sdate <- 1:length(sdate)
  dim(sdate) <- length(sdate)
  metadata <- list(sdate = list(standard_name = paste(strtoi(sdate),
						      collapse=", "),
                                units = paste0('Init date')))
  attr(sdate, 'variables') <- metadata
  names(dim(sdate)) <- 'sdate'

  return(list(time_step=time_step, time=time, sdate=sdate))

}

get_latlon <- function(latitude, longitude) {
  ## TODO: Verify if this is needed or if it needs a change now that we are
  ## using s2dv_cube objects
  # Adds dimensions and metadata to lat and lon
  # latitude: array containing the latitude values
  # longitude: array containing the longitude values
  ## the inputs are simply the arrays with the values?
  dim(longitude) <- length(longitude)
  ## TODO: Extract metadata from s2dv_cube
  metadata <- list(longitude = list(units = 'degrees_east'))
  # Add metadata to attributes
  attr(longitude, 'variables') <- metadata
  # Add longitude dimension
  names(dim(longitude)) <- 'longitude'
  
  dim(latitude) <- length(latitude)
  ## TODO: Extract metadata from s2dv_cube
  metadata <- list(latitude = list(units = 'degrees_north'))
  # Add metadata to attributes
  attr(latitude, 'variables') <- metadata
  # Add latitude dimension
  names(dim(latitude)) <- 'latitude'

  return(list(lat=latitude, lon=longitude))

}

## TODO: Place inside a function somewhere
# if (tolower(agg) == "country"){
#   load(mask.path)
#   grid <- europe.countries.iso
# } else {
#   grid <- list(lon=attr(var.obs, 'Variables')$dat1$longitude,
#                lat=attr(var.obs, 'Variables')$dat1$latitude)
# }

save_metrics <- function(skill,
			 recipe,
                         grid,
                         outfile,
                         leadtimes,
                         agg="global") {
  
  fcst.horizon <- tolower(recipe$Analysis$Horizon)
  fcst.sdate <- paste0(recipe$Analysis$Time$sdate$fcst_syear,
		       recipe$Analysis$Time$sdate$fcst_sday)
  

  lalo <- c('longitude', 'latitude')

  ## TODO: Sort out aggregation
  if (tolower(agg) == "global") {
    skill <-  lapply(skill, function(x){
                     Reorder(x[[1]], c(lalo, 'time'))})
  }

  for (i in 1:length(skill)) {

    metric <- names(skill[i])
    if (tolower(agg) == "country"){
      sdname <- paste0(names(metric), " region-aggregated metric")
      dims <- c('Country', 'time')
    } else {
      sdname <- paste0(names(metric), " grid point metric")
      dims <- c(lalo, 'time')
    }
    metadata <- list(name = metric, standard_name = sdname)

    attr(skill[i], 'variables') <- metadata        
    names(dim(skill[[i]])) <- dims 
  }

  times <- get_times(fcst.horizon, leadtimes, fcst.sdate)
  time <- times$time
  time_step <- times$time_step

 # if (tolower(agg) == "country") { 
 #   country <- get_countries(grid)
 #   ArrayToNc(append(country, time, skill, time_step), outfile)
 # } else  {

  latlon <- get_latlon(grid$lat, grid$lon)
  vars <- list(latlon$lat, latlon$lon, time)
  vars <- c(vars, skill, list(time_step))
  ArrayToNc(vars, outfile)

}

